//Licence MIT
//Created by Vitalii Rohozyn

#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library for ST7735
#include <Adafruit_ST7789.h> // Hardware-specific library for ST7789
#include <SPI.h>

#define AUTOCALIBRAION             // To start calibration or not
#define AUTOCALIBRATION_TIME  10   // Idk in what format it is, but basically for how long to take measuments for calibration
#define AUTOCALIBRATION_RATE  5    // how much time to delay between loops for calibrationg
#define RESISTOR_TRESHOLD_MIN 0    // Minimum treshold for resistor(doesn't affect anything if using calibration)
#define RESISTOR_TRESHOLD_MAX 1023 // Maximum treshold
#define CIRCLE_RADIUS_MIN     0    // Minimum circle Radius(For now takes only maximum)
#define CIRCLE_RADIUS_MAX     7
#define TRANS_SPEED           5    // Maximum delta movement for circble

#define RIGHTDISPLAY // If defined enables second display
//#define RIGHTONLY    // If defined disables first display

#define VOID ST77XX_WHITE // Defines color to be used as eraser
#define BASE ST77XX_BLACK // Defines color to be used to darken the displays

#ifdef RIGHTONLY
#define RIGHTDISPLAY
#endif

#if !defined(RIGHTONLY) // Defining pins for left display
#define TFT1_MOSI       13  // Data out
#define TFT1_SCLK       12  // Clock out
#define TFT1_CS         11  
#define TFT1_DC         10  
#define TFT1_RST        -1 // Or set to -1 and connect to Arduino RESET pin

Adafruit_ST7735 tft1 = Adafruit_ST7735(TFT1_CS, TFT1_DC, TFT1_MOSI, TFT1_SCLK, TFT1_RST);
#endif

#ifdef RIGHTDISPLAY // Defining pins for right display
#define TFT2_MOSI       9  // Data out
#define TFT2_SCLK       8  // Clock out
#define TFT2_CS         7
#define TFT2_DC         6
#define TFT2_RST        -1 // Or set to -1 and connect to Arduino RESET pin

Adafruit_ST7735 tft2 = Adafruit_ST7735(TFT2_CS, TFT2_DC, TFT2_MOSI, TFT2_SCLK, TFT2_RST);
#endif

int x_prev_left           = 0;
int y_prev_left           = 0;
int x_prev_right          = 0;
int y_prev_right          = 0;
int rad_prev              = 0;

#ifdef AUTOCALIBRAION
int resistor_treshold_min = 0;
int resistor_treshold_max = 0;
#else
int resistor_treshold_min = RESISTOR_TRESHOLD_MIN;
int resistor_treshold_max = RESISTOR_TRESHOLD_MAX;
#endif

void setup(void) {
  pinMode(A0, INPUT);
  pinMode(A1, INPUT);
  pinMode(A2, INPUT);
  pinMode(A3, INPUT);
  Serial.begin(9600);
  
  #ifdef AUTOCALIBRAION
  int delaytime = AUTOCALIBRATION_RATE;
  for (int i = 0; i < AUTOCALIBRATION_TIME * 10; i++) {
    int min_curr = resistor_treshold_min;
    int max_curr = resistor_treshold_max;
    for (int o = 0; o <= 3; o++) {
      int curr = 1023 - analogRead(o);
      Serial.println(curr);
      if (o == 0) {
        min_curr = curr;
        max_curr = curr;
        resistor_treshold_min = min_curr;
        resistor_treshold_max = max_curr;
        continue;
      };
      if (min_curr > curr) {
        min_curr = curr;
      }
      if (max_curr < curr) {
        max_curr = curr;
      };
    };
    if (resistor_treshold_min > min_curr) {
      resistor_treshold_min = min_curr;
    };
    if (resistor_treshold_max < max_curr) {
      resistor_treshold_max = max_curr;
    };
    delay(AUTOCALIBRATION_RATE);
  };
  Serial.println(resistor_treshold_min);
  Serial.println(resistor_treshold_max);
  #endif AUTOCALIBRAION

  #if !defined(RIGHTONLY)
    tft1.initR(INITR_GREENTAB);      // Init ST7735S chip, green tab
    Serial.println(F("First display initialized"));
    tft1.invertDisplay(false);
    tft1.fillScreen(VOID);
  #endif
  
  
  #ifdef RIGHTDISPLAY
    tft2.initR(INITR_GREENTAB);      // Init ST7735S chip, green tab
    Serial.println(F("Second display initialized"));
    tft2.invertDisplay(false);
    tft2.fillScreen(VOID);
  #endif

  delay(100);  
}

void loop() {
  int BOTTOMLEFT  = 1023 - analogRead(A3);
  int TOPLEFT     = 1023 - analogRead(A2);
  int BOTTOMRIGHT = 1023 - analogRead(A1);
  int TOPRIGHT    = 1023 - analogRead(A0);
  //Serial.println(BOTTOMLEFT);
  int BOTTOM = mymap((BOTTOMLEFT + BOTTOMRIGHT)/2, resistor_treshold_min, resistor_treshold_max, 0, 80); // IN REALITY IT'S RIGHT FOR LEFT EYE AND LEFT FOR RIGHT EYE
  int TOP    = mymap((TOPRIGHT   + TOPLEFT    )/2, resistor_treshold_min, resistor_treshold_max, 0, 80); // IN REALITY IT'S LEFT FOR LEFT EYE AND RIGHT FOR RIGHT EYE
  int RIGHT  = mymap((TOPRIGHT   + BOTTOMRIGHT)/2, resistor_treshold_min, resistor_treshold_max, 0, 64); // IN REALITY IT'S BOTTOM FOR LEFT EYE AND TOP FOR RIGHT EYE
  int LEFT   = mymap((BOTTOMLEFT + TOPLEFT    )/2, resistor_treshold_min, resistor_treshold_max, 0, 64); // IN REALITY IT'S TOP FOR LEFT EYE AND BOTTOM FOR RIGHT EYE
  //int rad = map((BOTTOM + TOP + RIGHT + LEFT)/4, 0, 1023 - resistor_treshold_min, CIRCLE_RADIUS_MIN, CIRCLE_RADIUS_MAX);
  int rad = CIRCLE_RADIUS_MAX;
  #if !defined(RIGHTONLY) // DRAWING FOR LEFT EYE
    int x_fin_left = (tft1.width()/2 - RIGHT) + (RIGHT + LEFT)/2;
    int y_fin_left = (tft1.height()/2 - BOTTOM) + (TOP + BOTTOM)/2;
    x_fin_left = plump(x_prev_left, x_fin_left, TRANS_SPEED);
    y_fin_left = plump(y_prev_left, y_fin_left, TRANS_SPEED);
    tft1.fillCircle(
      x_prev_left,
      y_prev_left,
      rad_prev,
      VOID
    );
    tft1.fillCircle(
      x_fin_left,
      y_fin_left,
      rad,
      BASE 
    );
  x_prev_left = x_fin_left;
  y_prev_left = y_fin_left;
  #endif
 
  #ifdef RIGHTDISPLAY // DRAWING FOR RIGHT EYE
    int x_fin_right = (tft2.width()/2 - LEFT) + (RIGHT + LEFT)/2;
    int y_fin_right = (tft2.height()/2 - TOP) + (TOP + BOTTOM)/2;
    x_fin_right = plump(x_prev_right, x_fin_right, TRANS_SPEED);
    y_fin_right = plump(y_prev_right, y_fin_right, TRANS_SPEED);
    tft2.fillCircle(
      x_prev_right,
      y_prev_right,
      rad_prev,
      VOID
    );
    tft2.fillCircle(
      x_fin_right,
      y_fin_right,
      rad,
      BASE 
    );
  x_prev_right = x_fin_right;
  y_prev_right = y_fin_right;  
  #endif
  rad_prev = rad;
  delay(5);
}

int plump(int prev, int next, int max) {
  if (abs(next - prev) > max) {
    if (prev > next){
      return next + max;
    } else {
      return next - max;
    }
  } else {
    return next;
  }
}

long mymap(double x, long in_min, long in_max, long out_min, long out_max) {
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}
